package me.Demon.BlockWars.Manager;

import com.sk89q.worldedit.EditSession;
import com.sk89q.worldedit.WorldEdit;
import com.sk89q.worldedit.bukkit.BukkitAdapter;
import com.sk89q.worldedit.math.BlockVector3;
import com.sk89q.worldedit.regions.CuboidRegion;
import me.Demon.BlockWars.Game.BlockRegion;
import me.Demon.BlockWars.Game.GameData;
import me.Demon.BlockWars.Game.Point;
import me.Demon.BlockWars.Game.Region;
import me.Demon.BlockWars.Main;
import me.Demon.BlockWars.Util.ComputeBlock;
import me.Demon.BlockWars.Util.GameUtil;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;

import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;

public abstract class MinerOperate {

    /*
    * 初始化矿区大小
    * */
    public static void initializeMinerSize() {
        int min_minerY = 10;
        int max_minerY = 18;
        GameData gameData = Main.gameData;
        Region region = new Region(Bukkit.getWorld("world"),new Point(103,min_minerY,-6),new Point(113, max_minerY, 4));
        gameData.setRegion(region);
        GameUtil.loadSchematics("miner_11x11"); // 初始化矿区模板
        gameData.setCompleteBlockGoal(ComputeBlock.getCompleteBlockGoal(region));
    }
    public static void clearMinerAll(List<BlockRegion> blockRegionList){
        World world = Main.gameData.getWorld();
        Material material = Material.AIR;
        for (BlockRegion blockRegion : blockRegionList) {
            CompletableFuture.runAsync(() -> {
                long startTime = System.currentTimeMillis(); // 记录开始时间
                int minX = blockRegion.getMinX();
                int maxX = blockRegion.getMaxX();
                CuboidRegion cuboidRegion = ComputeBlock.getCuboidRegion(minX,maxX);
                try (EditSession editSession = WorldEdit.getInstance().newEditSession(BukkitAdapter.adapt(world))) {
                    // 在编辑会话中设置长方体区域内的所有方块为指定的材质
                    editSession.setBlocks((com.sk89q.worldedit.regions.Region) cuboidRegion, BukkitAdapter.asBlockType(material));
                } catch (Exception e) {
                    // 捕获并打印任何可能发生的异常
                    e.printStackTrace();
                }
                if(Main.Debug) {
                    long endTime = System.currentTimeMillis(); // 记录结束时间
                    long duration = endTime - startTime; // 计算耗时
                    System.out.println("[调试 - 清理] 本次清理所有方块总计耗时 " + duration + " ms");
                }
            });
        }
        Bukkit.getScheduler().runTaskLater(Main.plugin, () -> {
            for (Player player : Bukkit.getOnlinePlayers()){
                player.teleport(Main.gameData.getRegion().getHub());
            }
            Main.gameData.refreshTheNonBlankBlockOfMiningArea(); // 刷新矿区当前块数量
        },20L);
    }

    public static void fillMinerAll(List<BlockRegion> blockRegionList){
        World world = Main.gameData.getWorld();
        Material material = Material.DIAMOND_BLOCK;
        for (BlockRegion blockRegion : blockRegionList) {
            CompletableFuture.runAsync(() -> {
                long startTime = System.currentTimeMillis(); // 记录开始时间
                int minX = blockRegion.getMinX();
                int maxX = blockRegion.getMaxX();
                CuboidRegion cuboidRegion = ComputeBlock.getCuboidRegion(minX,maxX);
                try (EditSession editSession = WorldEdit.getInstance().newEditSession(BukkitAdapter.adapt(world))) {
                    // 在编辑会话中设置长方体区域内的所有方块为指定的材质
                    editSession.setBlocks((com.sk89q.worldedit.regions.Region) cuboidRegion, BukkitAdapter.asBlockType(material));
                } catch (Exception e) {
                    // 捕获并打印任何可能发生的异常
                    e.printStackTrace();
                }
                if(Main.Debug) {
                    long endTime = System.currentTimeMillis(); // 记录结束时间
                    long duration = endTime - startTime; // 计算耗时
                    System.out.println("[调试 - 修复] 本次修复所有方块总计耗时 " + duration + " ms");
                }
            });
        }
        Bukkit.getScheduler().runTaskLater(Main.plugin, () -> {
            for (Player player : Bukkit.getOnlinePlayers()){
                player.teleport(Main.gameData.getRegion().getHub());
            }
            Main.gameData.refreshTheNonBlankBlockOfMiningArea(); // 刷新矿区当前块数量
        },20L);
    }

    public static void repairBlock(){
        long startTime = System.currentTimeMillis(); // 记录开始时间
        GameData game = Main.gameData;
        Region region = game.getRegion();
        boolean ford = false;
        Material material = Material.DIAMOND_BLOCK;// 获取当前层数下一层的方块类型
        int fillY = (int) region.getMin().getY(); // 获取修复的指定层数
        // 通过遍历区域中所有方块获取到高度
        for (int y = fillY; y <= region.getMax().getY(); y++) {
            if(ford){break;}
            for (int x = (int) Math.floor(region.getMin().getX()); x <= region.getMax().getX(); x++) {
                if(ford){break;}
                for (int z = (int) Math.floor(region.getMin().getZ()); z <= region.getMax().getZ(); z++) {
                    Block block = region.getWorld().getBlockAt(x, y, z);
                    if (block.getType().equals(Material.AIR)) {
                        fillY = y;
                        Block lowerBlock = block.getLocation().add(0,-1,0).getBlock();
                        material = getMaterial(lowerBlock.getType());
                        ford = true;
                        break;
                    }
                }
            }
        }
        BlockVector3 vector1 = BlockVector3.at(region.getMin().getX(), fillY, region.getMin().getZ());
        BlockVector3 vector2 = BlockVector3.at(region.getMax().getX(), fillY, region.getMax().getZ());
        CuboidRegion cuboidRegion = new CuboidRegion(BukkitAdapter.adapt(region.getWorld()), vector1, vector2);
        try (EditSession editSession = WorldEdit.getInstance().newEditSession(BukkitAdapter.adapt(region.getWorld()))) {
            editSession.setBlocks((com.sk89q.worldedit.regions.Region) cuboidRegion, BukkitAdapter.asBlockType(material));
        } catch (Exception e) {
            e.printStackTrace();
        }
        if(Main.Debug) {
            long endTime = System.currentTimeMillis(); // 记录结束时间
            long duration = endTime - startTime; // 计算耗时
            System.out.println("[调试 - 修复] 本次修复所有总计耗时 " + duration + " ms");
        }
    }
    
    public static Material getMaterial(Material material){
       if(material == Material.IRON_BLOCK) {
           return Material.GOLD_BLOCK;
       }else if(material == Material.GOLD_BLOCK) {
           return Material.DIAMOND_BLOCK;
       }else if(material == Material.DIAMOND_BLOCK) {
           return Material.EMERALD_BLOCK;
       }else if(material == Material.EMERALD_BLOCK) {
           return Material.IRON_BLOCK;
       }
       return Material.IRON_BLOCK;
    }
}
